/*
** Module   : SaxEntity
** Abstract : SAX attributes, reader, writer common interface.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130208 Created initial version.
** 002 EVL 20130911 Adding legacy attributes and methods annotation.
** 003 EVK 20131001 Added enum with input source.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface decribing the common features for SAX Reader, SAX Writer and SAX Attributes Progress
 * support objects.
 */
public interface SaxEntity
extends XCommon
{



   /**
    * List of possible 4gl input/output source types:
    * <ul>
    * <li>{@link #FILE} - from file</li>
    * <li>{@link #MEMPTR} - from memptr type</li>
    * <li>{@link #LONGCHAR} - from longchar type</li>
    * <li>{@link #STREAM} - from stream type</li>
    * <li>{@link #STREAM_HANDLE} - from stream-handle type</li>
    * </ul>
    */
   enum Source
   {
      FILE         ("FILE"),
      MEMPTR       ("MEMPTR"),
      LONGCHAR     ("LONGCHAR"),
      STREAM       ("STREAM"),
      STREAM_HANDLE("STREAM-HANDLE");

      /** 4gl name of input/output source */
      private final String name;

      Source(String name)
      {
         this.name = name;
      }

      /**
       * Returning the name of 4gl input/output source.
       *
       * @return   The 4gl name of input source.
       */
      @Override
      public String toString()
      {
         return name;
      }
   }
   // Attributes
   // Methods
   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(Text name, Text value, Text uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(String name, Text value, Text uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(Text name, String value, Text uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(Text name, Text value, String uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(String name, String value, Text uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(String name, Text value, String uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(Text name, String value, String uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(String name, String value, String uri);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(Text name, Text value);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(String name, Text value);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(Text name, String value);

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    * 
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "INSERT-ATTRIBUTE")
   public logical insertAttribute(String name, String value);
}
