/*
** Module   : SaxEntityImpl
** Abstract : SAX attributes, reader, writer common fatures iplementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130208 Created initial version. Implementation is limited to the stubs to get the
**                  converted java code compiled.
** 002 EVK 20130903 Moved valid() and unknown() amd removeAttribute() from XCommonImpl.
** 003 CA  20130927 Removed unknown() API, as by default the resource is known.
** 004 EVK 20131001 Implement methods and class logic. Added methods to process some SAX errors.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

import static com.goldencode.p2j.xml.SaxWriter.WriteStatus;

/**
 * Implementation the common features for SAX Reader, SAX Writer and SAX Attributes Progress
 * support objects.
 */
public class SaxEntityImpl
extends XCommonImpl
implements SaxEntity
{
   public static final int ERR_10512 = 10512;
   public static final int ERR_14586 = 14586;
   public static final int ERR_14607 = 14607;
   public static final int ERR_14609 = 14609;
   public static final int ERR_14613 = 14613;
   public static final int ERR_14630 = 14630;

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(String name, Text value, Text uri)
   {
      return insertAttribute(new character(name), value, uri);
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(Text name, String value, Text uri)
   {
      return insertAttribute(name, new character(value), uri);
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(Text name, Text value, String uri)
   {
      return insertAttribute(name, value, new character(uri));
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(String name, String value, Text uri)
   {
      return insertAttribute(new character(name), new character(value), uri);
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(String name, Text value, String uri)
   {
      return insertAttribute(new character(name), value, new character(uri));
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(Text name, String value, String uri)
   {
      return insertAttribute(name, new character(value), new character(uri));
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(String name, String value, String uri)
   {
      return insertAttribute(new character(name), new character(value), new character(uri));
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(String name, Text value)
   {
      return insertAttribute(new character(name), value, new character());
   }
   
   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(Text name, String value)
   {
      return insertAttribute(name, new character(value), new character());
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(String name, String value)
   {
      return insertAttribute(new character(name), new character(value), new character());
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(Text name, Text value)
   {
      return insertAttribute(name, value, new character());
   }

   /**
    * Adding the single attribute to the start tag in the XML document if applied for SAX writer
    * object. For SAX attributes object inserts attribute and its name into SAX attributes object.
    *
    * @param    name
    *           The attribute name to insert.
    * @param    value
    *           The attribute value to insert.
    * @param    uri
    *           The URI of the attribute to insert or '?' for unknown value.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertAttribute(Text name, Text value, Text uri)
   {
      // TODO:
      return new logical(true);
   }
   
   /**
    * Removes the specified attribute.
    *
    * @param name The name of the attribute to remove.
    * @param uri  The URI of the attribute to remove.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(character name, character uri)
   {
      return new logical(true);  //TODO autogenerated code by evk, please fix this
   }

   /**
    * Removes the specified attribute.
    *
    * @param index The 1-based index of the attribute to remove.
    * @param uri   The URI of the attribute to remove.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(NumberType index, character uri)
   {
      return new logical(true);  //TODO autogenerated code by evk, please fix this
   }

   /**
    * Reports if this object is valid for use.
    *
    * @return   <code>true</code> if we are valid (can be used).
    */
   @Override
   public boolean valid()
   {
      return true;
   }

   /**
    * Process and display mismatched parameters in callback.
    *
    * @param    method
    *           The read-only attribute.
    */
   protected void missmatchCallbackParameters(String method, String callbackName)
   {
      if (method == null)
      {
         throw new IllegalArgumentException("The method is null !");
      }
      String msg = "Callback error: %s sent procedure %s mismatched parameters";
      String err = String.format(msg, method.toUpperCase(), callbackName);
      ErrorManager.recordOrThrowError(ERR_10512, err, false);
   }

   /**
    * Process and display invalid read/write mode error. It displays appropriate error message.
    *
    * @param    attribute
    *           The read-only attribute.
    */
   protected void invalidReadWriteModeError(String attribute)
   {
      if (attribute == null)
      {
         throw new IllegalArgumentException("The attribute is null !");
      }
      String err = "Argument for SET-OUTPUT-DESTINATION must be a 'file', 'stream', " +
                   "'stream-handle','memptr', or 'longchar'";
      ErrorManager.recordOrShowError(ERR_14607, err, false, false, false);
   }

   /**
    * Process and display invalid status error. It displays appropriate error message.
    *
    * @param    attribute
    *           The read-only attribute.
    * @param    status
    *           Current status of SAX-WRITER.
    */
   protected void invalidStatusAttributeError(String attribute, WriteStatus status)
   {
      if (attribute == null)
      {
         throw new IllegalArgumentException("The attribute is null !");
      }
      String msg = "Error: setting attribute %s invalid while WRITE-STATUS is %s";
      String err = String.format(msg, attribute.toUpperCase(), status);
      ErrorManager.recordOrShowError(ERR_14630, err, false, false, false);
   }

   /**
    * Process and display invalid status error. It displays appropriate error message.
    *
    * @param    method
    *           The 4gl method name..
    */
   protected void writerOutputError(String method)
   {
      if (method == null)
      {
         throw new IllegalArgumentException("The method is null !");
      }
      String msg = "%s encountered an error with the output destination or encoding";
      String err = String.format(msg, method.toUpperCase());
      ErrorManager.recordOrShowError(ERR_14613, err, false, false, false);
   }

   /**
    * Process and display invalid status error. It displays appropriate error message.
    *
    * @param    method
    *           The 4gl method name.
    * @param    status
    *           Current status of SAX-WRITER.
    */
   protected void invalidStatusMethodError(String method, WriteStatus status)
   {
      if (method == null)
      {
         throw new IllegalArgumentException("The method is null !");
      }
      String msg = "%s method invalid while WRITE-STATUS is %s";
      String err = String.format(msg, method.toUpperCase(), status);
      ErrorManager.recordOrShowError(ERR_14609, err, false, false, false);
   }
}
