/*
** Module   : SaxReaderImpl
** Abstract : SAX reader specific features implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130208 Created initial version. Implementation is limited to the stubs to get the
**                  converted java code compiled.
** 002 CA  20130221 Added ADM-DATA and UNIQUE-ID support. 
** 003 EVL 20130320 Spelling fixes for comments.
** 004 EVK 20131001 Implement methods and class logic.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;
import org.xml.sax.*;
import org.xml.sax.ext.Locator2Impl;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.*;
import javax.xml.stream.*;
import java.io.*;

/**
 * Implementation SAX-Reader specific features of the SAX objects.
 */
public class SaxReaderImpl
extends SaxEntityImpl
implements SaxReader
{
   /** Current status of SAX-Writer */
   private ReadStatus readStatus = ReadStatus.SAX_UNINITIALIZED;

   /** Current input source of SAX-Reader */
   private Source source = null;

   /** Current input source of SAX-Reader */
   private InputStream inputStream = null;

   /** Sax-parser object */
   private SAXParser saxParser;

   /** Specify the location of the SAX callback procedures */
   private handle hCallback;

   /** validation flag */
   private boolean validate       = true;

   /** validation flag */
   private boolean suppress       = false;

   /** Schema paths locations variable */
   private String  schemaPaths    = null;

   /** Schema locations variable */
   private String  schemaLocation = "";

   /** Locator object for parser status maintenance */
   private Locator locator;

   /**
    * Default constructor.
    */
   public SaxReaderImpl()
   {
   }

   /**
    * Get the value of the ADM-DATA attribute.
    *
    * @return See above.
    */
   @Override
   public character getADMData()
   {
      // TODO Auto-generated method stub
      return null;
   }

   /**
    * Set the value of the ADM-DATA attribute.
    *
    * @param    value
    *           The new value.
    */
   @Override
   public void setADMData(character value)
   {
      // TODO Auto-generated method stub
   }

   /**
    * Gets the value of the handler atribute for the given SAX Reader object.
    *
    * @return The handle variable representing the handler attribute.
    */
   @Override
   public handle getHandler()
   {
      return new handle(hCallback);
   }

   /**
    * Sets the value of the handler atribute for the given SAX Reader object.
    *
    * @param    hCallback
    *           The handle of the new callback handler to be set for the handler attribute.
    */
   @Override
   public void setHandler(handle hCallback)
   {
      this.hCallback = new handle(hCallback);
   }
   
   /**
    * Gets the value of the current 1-based column in XML source.
    *
    * @return   The integer value of the current column.
    */
   @Override
   public integer getLocatorColumnNumber()
   {
      if (locator == null)
      {
         return new integer();
      }
      return new integer(locator.getColumnNumber());
   }

   /**
    * Gets the value of the current 1-based line number in XML source.
    *
    * @return   The integer value of the current line.
    */
   @Override
   public integer getLocatorLineNumber()
   {
      if (locator == null)
      {
         return new integer();
      }
      return new integer(locator.getLineNumber());
   }

   /**
    * Gets the current value of the public identifier of the current XML source.
    *
    * @return   The character value representing the current locator-public-id attribute.
    */
   @Override
   public character getLocatorPublicId()
   {
      if (locator == null)
      {
         return new character();
      }
      return new character(locator.getPublicId());
   }
   
   /**
    * Gets the current value of the system identifier of the current XML source.
    *
    * @return   The character value representing the current locator-system-id attribute.
    */
   @Override
   public character getLocatorSystemId()
   {
      if (locator == null)
      {
         return new character();
      }
      return new character(locator.getSystemId());
   }
   
   /**
    * Gets the current value of the parse status. The default value is
    * {@link ReadStatus#SAX_UNINITIALIZED}. The other returned values are
    * {@link ReadStatus#SAX_COMPLETE}, {@link ReadStatus#SAX_PARSER_ERROR} or
    * {@link ReadStatus#SAX_RUNNING}.
    *
    * @return   The integer value representing the current status of the parse.
    */
   @Override
   public integer getParseStatus()
   {
      return new integer(readStatus.getStatus());
   }
   
   /**
    * Gets the current value of the namespace location pair mapping.
    *
    * @return   The character value representing the current schema-location attribute.
    */
   @Override
   public character getSchemaLocation()
   {
      return new character(schemaLocation);
   }
   
   /**
    * Sets the current value of the namespace location pair mapping.
    *
    * @param    location
    *           The new value the current schema-location attribute.
    */
   @Override
   public void setSchemaLocation(String location)
   {
      setSchemaLocation(new character(location));
   }
   
   /**
    * Gets the delimiter separated list of directory paths used to locate XML DTD associated with
    * a particular XML document.
    *
    * @return   The character value representing the current schema-path attribute.
    */
   @Override
   public character getSchemaPath()
   {
      return new character(schemaPaths);
   }
   
   /**
    * Sets the delimiter separated list of directory paths used to locate XML DTD associated with
    * a particular XML document.
    * 
    * @param    path
    *           The new value the current schema-path attribute.
    */
   @Override
   public void setSchemaPath(String path)
   {
      setSchemaPath(new character(path));
   }
   
   /**
    * Indicates if the namespace processing is suppressed or not.
    * 
    * @return   The logical value representing the current suppress-namespace-processing
    *           attribute.
    */
   @Override
   public logical getSuppressNamespaceProcessing()
   {
      return new logical(suppress);
   }
   
   /**
    * Redefines whether the namespace processing is suppressed or not.
    *
    * @param    suppress
    *           The new value the current suppress-namespace-processing attribute.
    */
   @Override
   public void setSuppressNamespaceProcessing(boolean suppress)
   {
      setSuppressNamespaceProcessing(new logical(suppress));
   }
   
   /**
    * Gets the the unique ID number associated to this object by the underlying system. Not the
    * same as the handle value itself.
    *
    * @return   The integer number of the children.
    */
   @Override
   public integer getUniqueID()
   {
      // TODO Auto-generated method stub
      return null;
   }
   
   /**
    * Indicates if the parser validates the XML document against DTD or not.
    * 
    * @return   The logical value representing the current validation-enabled attribute.
    */
   @Override
   public logical getValidationEnabled()
   {
      return new logical(validate);
   }
   
   /**
    * Redefines whether the parser will validate the XML document against DTD or not.
    *
    * @param    enable
    *           The new value the current validation-enabled attribute.
    */
   @Override
   public void setValidationEnabled(boolean enable)
   {
      setValidationEnabled(new logical(enable));
   }
   
   /**
    * Executing single call parse of an XML document associated with a SAX-Reader object.
    */
   @Override
   public void saxParse()
   {
      SAXParserFactory factory = SAXParserFactory.newInstance();
      try
      {
         saxParser = factory.newSAXParser();
         DefaultHandler handler   = new ProcDefaultHandler(hCallback, new handle(this));
         locator = new Locator2Impl();
         handler.setDocumentLocator(locator);

         SAXParserFactory spf = SAXParserFactory.newInstance();
         spf.setNamespaceAware(true);
         spf.setValidating(suppress || validate);
         saxParser = spf.newSAXParser();
         saxParser.parse(inputStream, handler);
      }
      catch (ParserConfigurationException | IOException ignored)
      {
         invalidArgumentError("PARSE");
      }
      catch (ProcDefaultHandler.MismatchedParameters e)
      {
   //Callback error: mismatched number of parameters passed to procedure Characters.(10511) todo
         missmatchCallbackParameters("PARSE", e.callbackName);
      }
      catch (SAXException ignored)
      {
         saxParseError("PARSE");
      }
   }
   
   // Methods

   /**
    * Initializes and begins progressive-scan parse of an XML document associated with a
    * SAX-Reader object.
    */
   @Override
   public void saxParseFirst()
   {

      // TODO:
      return;
   }

   /**
    * Continues progressive-scan parse of an XML document associated with a SAX-Reader object.
    */
   @Override
   public void saxParseNext()
   {
      // TODO:
      return;
   }

   /**
    * Set the value of the ADM-DATA attribute.
    *
    * @param    value
    *           The new value.
    */
   @Override
   public void setADMData(String value)
   {
      // TODO Auto-generated method stub
   }

   /**
    * Specifies the input source of the XML document to be parsed by a SAX-Reader object.
    * 
    * @param    mode
    *           The type of the object to process. Can be 'FILE', 'HANDLE', 'MEMPTR' or
    *           'LONGCHAR'.
    * @param    source
    *           The appropriate object to process.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical setInputSource(String mode, Object source)
   {
      return setInputSource(new character(mode), source);
   }

   /**
    * Specifies the input source of the XML document to be parsed by a SAX-Reader object.
    * 
    * @param    mode
    *           The type of the object to process. Can be 'FILE', 'HANDLE', 'MEMPTR' or
    *           'LONGCHAR'.
    * @param    source
    *           The appropriate object to process.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical setInputSource(character mode, Object source)
   {
      final String readMode;
      if (mode.isUnknown())
      {
         readMode = "file";
      }
      else
      {
         readMode = mode.getValue();
      }

      if ("file".equalsIgnoreCase(readMode))
      {
         this.source = Source.FILE;
         if (!(source instanceof character))
         {
            invalidArgumentError("SET-INPUT-SOURCE");
            return new logical(false);
         }
         inputStream = new FileInputStreamWrapper((String) source);
      }
      else if ("stream".equalsIgnoreCase(readMode))
      {
         this.source = Source.STREAM;
      }
      else if ("memptr".equalsIgnoreCase(readMode))
      {
         this.source = Source.MEMPTR;
         if (!(source instanceof memptr))
         {
            invalidArgumentError("SET-INPUT-SOURCE");
            return new logical(false);
         }
         inputStream = new MemptrInputStream((memptr) source);
      }
      else if ("stream-handle".equalsIgnoreCase(readMode))
      {
         this.source = Source.STREAM_HANDLE;
      }
      else if ("longchar".equalsIgnoreCase(readMode))
      {
         this.source = Source.LONGCHAR;
         if (!(source instanceof longchar))
         {
            invalidArgumentError("SET-INPUT-SOURCE");
            return new logical(false);
         }
         inputStream = new LongcharInputStream((longchar) source);
      }
      else
      {
         invalidReadWriteModeError("SET-INPUT-SOURCE");
         return new logical(false);
      }

      return new logical(true);
   }

   /**
    * Sets the current value of the namespace location pair mapping.
    *
    * @param    location
    *           The new value the current schema-location attribute.
    */
   @Override
   public void setSchemaLocation(character location)
   {
      if (location.isUnknown())
      {
         invalidArgumentError("SCHEMA-LOCATION");
      }
      schemaLocation = location.toStringMessage();
   }

   /**
    * Sets the delimiter separated list of directory paths used to locate XML DTD associated with
    * a particular XML document.
    *
    * @param    path
    *           The new value the current schema-path attribute.
    */
   @Override
   public void setSchemaPath(character path)
   {
      if (path.isUnknown())
      {
         invalidArgumentAssignError("SCHEMA-PATH");
         return;
      }

      schemaPaths = path.toStringMessage();
   }

   /**
    * Redefines whether the namespace processing is suppressed or not.
    *
    * @param    suppress
    *           The new value the current suppress-namespace-processing attribute.
    */
   @Override
   public void setSuppressNamespaceProcessing(logical suppress)
   {
      this.suppress = suppress.booleanValue();
   }

   /**
    * Redefines whether the parser will validate the XML document against DTD or not.
    *
    * @param    enable
    *           The new value the current validation-enabled attribute.
    */
   @Override
   public void setValidationEnabled(logical enable)
   {
      validate = enable.booleanValue();
   }

   /**
    * Causes parser to stop parsing the XML document.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical stopParsing()
   {
      readStatus = ReadStatus.SAX_COMPLETE;
      return new logical(true);
   }

   /**
    * Special wrapper to wrap read from longchar in lazy mode. It's means that first try to read
    * will check if longchar is available.
    */
   static class LongcharInputStream extends InputStream
   {
      /** A valid longchar object to read*/
      private longchar lchar;
      /** Lazy initialization flag.*/
      private boolean isStarted;
      /** Buffered input Stream.*/
      private ByteArrayInputStream bais;

      /**
       * Constructor accepting a longchar, which will read after first call.
       *
       * @param    source
       *           A valid longchar object.
       */
      LongcharInputStream(longchar source)
      {
         lchar = source;
         isStarted = false;
      }

      @Override
      public int read() throws IOException
      {
         if (!isStarted)
         {
            isStarted = true;
            bais = new ByteArrayInputStream(lchar.toStringMessage().getBytes());
         }
         return bais.read();
      }
   }

   /**
    * Special wrapper to wrap read from memptr in lazy mode. It's means that first try to read
    * will check if memptr is available.
    */
   private static class MemptrInputStream extends InputStream
   {
      /** A valid memptr */
      private final memptr mptr;
      /** Current read position starter from index 1.*/
      private long position;

      /**
       * Constructor accepting a memptr, which will read after first call.
       *
       * @param    source
       *           A valid memptr object.
       */
      private MemptrInputStream(memptr source)
      {
         mptr = source;
         position = 1;
      }

      @Override
      public int read() throws IOException
      {
         try
         {
            return mptr.getByte(position++).intValue();
         }
         catch (ErrorConditionException e)
         {
            throw new IOException(e);
         }
      }
   }


   /**
    * Special wrapper to wrap read from file in lazy mode. It's means that first try to read
    * will check if file is available.
    */
   static class FileInputStreamWrapper extends InputStream
   {
      /** File name.*/
      private final String fileName;
      /** Lazy initialization flag.*/
      private boolean isStarted;
      /** Buffered input Stream.*/
      private Stream stream;

      /**
       * Constructor accepting a file name, which will read after first call.
       *
       * @param    source
       *           A file name.
       */
      FileInputStreamWrapper(String source)
      {
         isStarted = false;
         fileName = source;
      }

      @Override
      public int read() throws IOException
      {
         try
         {
            if (!isStarted)
            {
               //TODO check errors and null
               stream = StreamFactory.openFileStream(fileName, false, false);
               isStarted = true;
            }

            return stream.readCh();
         }
         finally
         {
            if (stream != null)
            {
               stream.close();
            }
         }
      }
   }

   /**
    * Process and display mismatched parameters in callback.
    *
    * @param    method
    *           The read-only attribute.
    */
   private void saxParseError(String method)
   {
      if (method == null)
      {
         throw new IllegalArgumentException("The method is null !");
      }
      String msg = "SAX parser error: %s, Unexpected error";
      String err = String.format(msg, method.toUpperCase());
      ErrorManager.recordOrThrowError(ERR_14586, err, false);
   }
}
